@@ -501,3 +501,129 @@ class PorticoPreservation(models.Model):
 
                 logging.info(f'Added {row["Title"]} to '
                              f'{PorticoPreservation.name()} data')
+
+
+class OculScholarsPortalPreservation(models.Model):
+    class Meta:
+        db_table = "preservationData_oculpreservation"
+
+    title = models.TextField()
+
+    """
+    ONIX field codes to identify an ISSN:
+    ResourceVersionIDType = 7
+    IDValue = unhyphenated ISSN
+    
+    e.g.
+    <oph:ResourceVersion>
+        <oph:ResourceVersionIdentifier>
+        <oph:ResourceVersionIDType>07</oph:ResourceVersionIDType>
+        <oph:IDValue> 27697541</oph:IDValue>
+    </oph:ResourceVersionIdentifier>
+    """
+    issn = models.CharField(max_length=20)
+    preserved_volumes = models.TextField()
+    preserved_no = models.TextField()
+
+    # this indicates whether the title is preserved or queued
+    """
+    ONIX field codes for the <oph:PreservationStatus><oph:PreservationStatusCode> tags:
+    00	Unknown	The preservation status is genuinely unknown, and should be updated as soon as information is available	5		
+    01	Will not be preserved	Preservation agency has decided against preservation of the issues and e-content in the <Coverage> statement	5		
+    02	Undecided	Preservation agency is considering preservation of the issues and e-content in the <Coverage> statement. No decision has yet been made. This status should be updated as soon as information is available	5		
+    03	Committed	Preservation agency is committed to preserving the issues and e-content in the <Coverage> statement. No active steps have yet been taken	5		
+    04	In progress	Preservation agency is in the process of preserving the issues and e-content in the <Coverage> statement	5		
+    05	Preserved	Preservation agency has preserved the issues and e-content in the <Coverage> statement	5
+    """
+    preserved = models.IntegerField()
+
+    """
+    ONIX field codes for the <oph:VerificationStatus> tag:
+    00	Unknown	It is genuinely unknown whether the preservation agency has checked the preserved holdings in the <Coverage> statement. This status should be updated when information is available		
+    01	Unverified	Preservation agency has not checked the preserved holdings in the <Coverage> statement, and it is not known whether all issues or e-content items are preserved
+    02	Verification in progress	Preservation agency is in the process of checking the preserved holdings in the <Coverage> statement, and nothing can be said about completeness at this stage		
+    03	Verified and incomplete	Preservation agency has checked the preserved holdings in the <Coverage> statement, and some issues or e-content items intended to be preserved are not complete		
+    04	Verified and complete	Preservation agency has checked the preserved holdings in the <Coverage> statement, and all issues or e-content items intended to be preserved are complete
+    """
+    verified = models.IntegerField()
+
+    publisher = models.ForeignKey(Publisher, on_delete=models.CASCADE,
+                                  default=None)
+
+    @staticmethod
+    def name():
+        return 'OCUL Scholars Portal'
+
+    @staticmethod
+    def preservation(container_title, issn, volume, no=None):
+        """
+        Determine whether a DOI is preserved in Portico
+        :param container_title: the container title
+        :param issn: the ISSN
+        :param volume: the volume
+        :param no: the issue number
+        :return: A PorticoPreservation item (or None) and a bool indicating
+        whether the item is fully preserved
+        """
+        preserved_item = utils.get_preserved_item_record(PorticoPreservation,
+                                                         container_title, issn)
+
+        if not preserved_item or len(preserved_item) == 0:
+            return None, False
+
+        # Portico gives volume formats as follows:
+        # 2013/2014 - v. 2 (1-2)
+        volume_regex = r'v\.\s(\d+)'
+
+        for pi in preserved_item:
+            matches = re.findall(volume_regex, pi.preserved_volumes)
+
+            volume = str(volume)
+
+            if volume in matches:
+                return pi, pi.preserved
+
+        return None, False
+
+    @staticmethod
+    @transaction.atomic
+    def import_data(url, bucket="", s3client=None):
+        # download the data file from S3 bucket
+        xml_file = utils.download_remote(False, OculScholarsPortalPreservation,
+                                         url, bucket=bucket, s3client=s3client,
+                                         decode=False)
+
+        # clear out
+        utils.clear_out(OculScholarsPortalPreservation)
+
+        fields = ['oph:ResourceVersionIDType', 'oph:IDValue',
+                  'oph:FixedCoverage', 'oph:PreservationStatusCode',
+                  'oph:VerificationStatus']
+
+        xml_parsed = utils.process_onix(
+            xml_file, fields,
+            callback=OculScholarsPortalPreservation.create_preservation)
+
+        for ocul_item in xml_parsed:
+            print(ocul_item)
+
+    @staticmethod
+    def create_preservation(output) -> None:
+        """
+        Create a preservation item of this model
+        :param output: a dictionary from an ONIX import
+        :return: None
+        """
+
+        publisher, created = \
+            Publisher.objects.get_or_create(name=output['publisher'])
+
+        OculScholarsPortalPreservation.objects.create(
+            issn=output['issn'], title=output['title'],
+            preserved_volumes=output['volume'],
+            preserved_no=output['issue'], preserved=output['status'],
+            verified=output['verification'], publisher=publisher
+        )
+
+        logging.info(f'Added {output["title"]} to '
+                     f'{OculScholarsPortalPreservation.name()} data')
