@@ -7,6 +7,10 @@ import requests
 from crossref.restful import Etiquette, Works
 from django.db.models import QuerySet
 
+from lxml import etree as ET
+import xmltodict
+from rich.progress import track
+
 
 def show_preservation(container_title: str, issn: str, volume: str,
                       no: str | None, doi: str,
@@ -152,10 +156,14 @@ def clear_out(model):
     model.objects.all().delete()
 
 
-def download_remote(local, model, url, bucket="", s3client=None):
+def download_remote(local, model, url, bucket="", s3client=None, decode=True):
     if s3client:
-        return s3client.get_object(Bucket=bucket, Key=url)[
-            'Body'].read().decode('utf-8')
+        if decode:
+            return s3client.get_object(Bucket=bucket, Key=url)[
+                'Body'].read().decode('utf-8')
+        else:
+            return s3client.get_object(Bucket=bucket, Key=url)[
+                'Body'].read()
 
     if not local:
         logging.info(f'Downloading: {model.name()} data')
@@ -163,8 +171,13 @@ def download_remote(local, model, url, bucket="", s3client=None):
     else:
         logging.info(f'Using local file for {model.name()} data')
 
-        with open(url, 'r') as f:
-            csv_file = f.read()
+        if decode:
+            with open(url, 'r') as f:
+                csv_file = f.read()
+        else:
+            with open(url, 'rb') as f:
+                csv_file = f.read()
+
     return csv_file
 
 
@@ -239,3 +252,84 @@ def get_preserved_item_record(model, container_title, issn) -> QuerySet | None:
                 return None
 
     return preserved_item
+
+
+def process_onix(xml_file, elements, callback):
+    """
+    A faster method for processing ONIX XML than using DOM methods
+    """
+    root = ET.XML(xml_file)
+
+    outputs = []
+
+    for holdings_record in root.findall(".//{http://www.editeur.org/onix/serials/SOH}HoldingsRecord"):
+
+        resource_versions = holdings_record.findall(".//{http://www.editeur.org/onix/serials/SOH}ResourceVersion")
+        for resource_version in resource_versions:
+
+            resource_version_identifiers = resource_version.findall(".//{http://www.editeur.org/onix/serials/SOH}ResourceVersionIdentifier")
+            for resource_version_identifier in resource_version_identifiers:
+
+                resource_type = resource_version_identifier.find(".//{http://www.editeur.org/onix/serials/SOH}ResourceVersionIDType")
+
+                # determine if ISSN
+                if int(resource_type.text) == 7:
+                    # extract unhyphenated ISSN, title, and publisher
+                    issn = resource_version_identifier.find(".//{http://www.editeur.org/onix/serials/SOH}IDValue").text
+                    title = resource_version.find(".//{http://www.editeur.org/onix/serials/SOH}TitleText").text
+                    publisher = resource_version.find(".//{http://www.editeur.org/onix/serials/SOH}PublisherName").text
+
+                    volume = None
+                    issue = None
+
+                    # extract volume and issue
+                    coverages = resource_version.findall(".//{http://www.editeur.org/onix/serials/SOH}Coverage")
+
+                    for coverage in coverages:
+                        level_one = coverage.find(".//{http://www.editeur.org/onix/serials/SOH}Level1")
+                        level_two = coverage.find(".//{http://www.editeur.org/onix/serials/SOH}Level2")
+
+                        levels = [level_one, level_two]
+
+                        for level in levels:
+                            unit = level.find(".//{http://www.editeur.org/onix/serials/SOH}Unit").text
+                            number = level.find(".//{http://www.editeur.org/onix/serials/SOH}Number").text
+
+                            if unit == 'Volume':
+                                volume = number
+                            elif unit == 'Issue':
+                                issue = number
+                            else:
+                                print('Unrecognised unit: {}'.format(unit))
+
+                    # preservation and verification status
+                    try:
+                        status = resource_version.find(".//{http://www.editeur.org/onix/serials/SOH}PreservationStatusCode").text
+                        verification = resource_version.find(".//{http://www.editeur.org/onix/serials/SOH}VerificationStatus").text
+
+                        output = {'issn': normalize_issn(issn), 'title': title,
+                                  'publisher': publisher, 'volume': volume,
+                                  'issue': issue, 'status': status,
+                                  'verification': verification}
+
+                        outputs.append(output)
+
+                        if callback:
+                            callback(output)
+
+                    except AttributeError:
+                        # if we get here, there is no preservation or
+                        # verification status
+                        # print("ERROR on {}".format(issn))
+                        pass
+
+    return outputs
+
+
+def normalize_issn(issn) -> str:
+    """
+    Normalizes an ISSN
+    :param issn: the ISSN
+    :return: the normalized ISSN
+    """
+    return f'{issn[0:4]}-{issn[4:8]}' if '-' not in issn else issn
